@@ -71,7 +71,7 @@ pub async fn new(
     // create a new repository in the directory specified by the project name
     create_git_repo(jetty_config.get_name())?;
     // add schemas and vs code settings
-    let jetty = &new_jetty_with_connectors(jetty_config.get_name()).await?;
+    let jetty = &new_jetty_with_connectors(jetty_config.get_name(), false).await?;
     write::config::write_settings_and_schema(jetty, jetty_config.get_name())?;
     Ok(())
 }
@@ -84,7 +84,7 @@ fn create_git_repo<P: AsRef<Path>>(project_path: P) -> Result<()> {
             .context("updating name of branch to main")?;
         std::fs::write(
             PathBuf::from(project_path.as_ref()).join(".gitignore"),
-            "# Environment state data\n.data/",
+            "# Environment state data\n.data/\n",
         )?;
 
         Ok(())
@@ -219,7 +219,7 @@ async fn initialize_project_structure(
 
     println!("\n\nCongratulations, your jetty project has been created and configured!");
     println!("To get started, run the following commands:\n");
-    println!("\t$ cd {}", project_path);
+    println!("\t$ cd {project_path}");
     println!("\t$ jetty bootstrap\n\n");
     Ok(())
 }
