@@ -13,25 +13,21 @@ use jetty_core::connectors::AssetType;
 use jetty_core::connectors::UserIdentifier;
 
 use jetty_core::connectors::nodes::ConnectorData;
-use jetty_core::connectors::nodes::EffectivePermission;
 use jetty_core::connectors::nodes::RawPolicy;
 use jetty_core::connectors::nodes::RawPolicyGrantee;
-use jetty_core::connectors::nodes::SparseMatrix;
 use jetty_core::cual::Cualable;
 use jetty_core::logging::debug;
 use jetty_core::logging::error;
-use jetty_core::permissions::matrix::InsertOrMerge;
+use jetty_core::print_runtime;
 
 use super::cual::{self, cual, get_cual_account_name, Cual};
 use crate::consts::DATABASE;
 use crate::consts::SCHEMA;
 use crate::consts::TABLE;
 use crate::consts::VIEW;
-use crate::efperm::EffectivePermissionMap;
 use crate::entry_types;
 use crate::entry_types::ObjectKind;
 use crate::entry_types::RoleName;
-use crate::Asset;
 use crate::FutureGrant;
 use crate::Grant;
 use crate::GrantType;
@@ -62,7 +58,7 @@ pub(super) struct Coordinator<'a> {
     pub(crate) role_grants: HashMap<Grantee, HashSet<RoleName>>,
 }
 
-#[derive(Hash, Eq, PartialEq)]
+#[derive(Hash, Eq, PartialEq, Debug)]
 pub(crate) enum Grantee {
     User(String),
     Role(String),
@@ -108,12 +104,13 @@ impl<'a> Coordinator<'a> {
             hold.push(Box::pin(self.conn.get_objects_futures(schema, m)));
         }
 
-        // for each role, get grants to that role
+        // Get all the object grants
         let grants_to_role_mutex = Arc::new(Mutex::new(&mut self.env.standard_grants));
-        for role in &self.env.roles {
-            let m = Arc::clone(&grants_to_role_mutex);
-            hold.push(Box::pin(self.conn.get_grants_to_role_future(role, m)));
-        }
+        let grants_to_role_mutex_clone = Arc::clone(&grants_to_role_mutex);
+        hold.push(Box::pin(
+            self.conn
+                .get_privilege_grants_future(grants_to_role_mutex_clone),
+        ));
 
         // for each role, get grants of
         let target_arc = Arc::new(Mutex::new(&mut self.env.role_grants));
@@ -131,7 +128,7 @@ impl<'a> Coordinator<'a> {
             ));
         }
 
-        // for database, get future grants, using the same Arc<Mutex>
+        // for database, get future grants, using the same Arc<Mutex> as used for schemas
         for database in &self.env.databases {
             let m = Arc::clone(&future_grants_arc);
             hold.push(Box::pin(
@@ -154,16 +151,23 @@ impl<'a> Coordinator<'a> {
 
         let mut connector_data = nodes::ConnectorData {
             // 19 Sec
-            groups: self.get_jetty_groups(),
+            groups: print_runtime!("getting jetty groups", self.get_jetty_groups()),
             // 7 Sec
-            users: self.get_jetty_users(),
+            users: print_runtime!("getting jetty users", self.get_jetty_users()),
             // 3.5 Sec
-            assets: self.get_jetty_assets(),
-            tags: self.get_jetty_tags(),
-            policies: self.get_jetty_policies(),
-            default_policies: self.get_jetty_default_policies(),
-            effective_permissions: self.get_effective_permissions(),
-            asset_references: Default::default(),
+            assets: print_runtime!("getting jetty assets", self.get_jetty_assets()),
+            tags: print_runtime!("getting jetty tags", self.get_jetty_tags()),
+            policies: print_runtime!("getting jetty policies", self.get_jetty_policies()),
+            default_policies: print_runtime!(
+                "getting jetty default_policies",
+                self.get_jetty_default_policies()
+            ),
+            effective_permissions: print_runtime!(
+                "getting jetty effective_permissions",
+                // self.get_effective_permissions()
+                Default::default()
+            ),
+            asset_references: print_runtime!("getting jetty asset_references", Default::default()),
             cual_prefix: Some(
                 cual::get_cual_prefix()
                     .context("cual account not yet set")
@@ -392,53 +396,6 @@ impl<'a> Coordinator<'a> {
         }
         res
     }
-
-    /// get effective_permissions from environment
-    pub(crate) fn get_effective_permissions(
-        &self,
-    ) -> SparseMatrix<String, Cual, HashSet<EffectivePermission>> {
-        let mut res = HashMap::new();
-        let ep_map = EffectivePermissionMap::new(&self.role_grants);
-
-        // The runtime performance here can definitely be improved, but this is
-        // a workable naive approach for now.
-        for user in &self.env.users {
-            let mut obj_eps = HashMap::new();
-            for obj in &self.env.objects {
-                obj_eps.insert_or_merge(
-                    obj.cual(),
-                    ep_map.get_effective_permissions_for_object(&self.env, user, obj),
-                );
-            }
-            res.insert_or_merge(user.name.to_owned(), obj_eps);
-            let mut db_eps = HashMap::new();
-            for db in &self.env.databases {
-                db_eps.insert_or_merge(
-                    db.cual(),
-                    ep_map.get_effective_permissions_for_asset(
-                        &self.env,
-                        user,
-                        &Asset::Database(db.clone()),
-                    ),
-                );
-            }
-            res.insert_or_merge(user.name.to_owned(), db_eps);
-            let mut schema_eps = HashMap::new();
-            for schema in &self.env.schemas {
-                schema_eps.insert_or_merge(
-                    schema.cual(),
-                    ep_map.get_effective_permissions_for_asset(
-                        &self.env,
-                        user,
-                        &Asset::Schema(schema.clone()),
-                    ),
-                )
-            }
-            res.insert_or_merge(user.name.to_owned(), schema_eps);
-        }
-
-        res
-    }
 }
 
 /// This function adds empty privileges to all existing objects that don't have the default privileges that would be applied if there
