@@ -1,5 +1,4 @@
 use serde::{Deserialize, Serialize};
-
 /// A type of object.
 #[derive(Copy, Clone, Default, Deserialize, Serialize, Debug)]
 pub enum ObjectKind {
@@ -9,6 +8,15 @@ pub enum ObjectKind {
     #[serde(rename = "VIEW")]
     View,
 }
+
+impl ToString for ObjectKind {
+    fn to_string(&self) -> String {
+        match self {
+            ObjectKind::Table => "TABLE".to_string(),
+            ObjectKind::View => "VIEW".to_string(),
+        }
+    }
+}
 /// Snowflake Table entry.
 #[derive(Clone, Default, Deserialize, Serialize, Debug)]
 pub struct Object {
@@ -23,8 +31,4 @@ impl Object {
     pub(crate) fn fqn(&self) -> String {
         format!("{}.{}.{}", self.database_name, self.schema_name, self.name)
     }
-
-    pub(crate) fn schema_fqn(&self) -> String {
-        format!("{}.{}", self.database_name, self.schema_name)
-    }
 }
