@@ -18,7 +18,6 @@ mod consts;
 mod coordinator;
 mod creds;
 mod cual;
-mod efperm;
 mod entry_types;
 mod rest;
 mod write;
@@ -26,7 +25,7 @@ mod write;
 use cual::set_cual_account_name;
 pub use entry_types::{
     Asset, Database, Entry, FutureGrant, Grant, GrantOf, GrantType, Object, Role, RoleName, Schema,
-    StandardGrant, Table, User, View, Warehouse,
+    StandardGrant, User, Warehouse,
 };
 use futures::StreamExt;
 use jetty_core::access_graph::translate::diffs::LocalConnectorDiffs;
@@ -34,7 +33,7 @@ use jetty_core::connectors::{
     AssetType, ConnectorCapabilities, NewConnector, ReadCapabilities, WriteCapabilities,
 };
 use jetty_core::jetty::ConnectorManifest;
-use jetty_core::logging::error;
+use jetty_core::logging::{debug, error};
 
 use rest::{SnowflakeRequestConfig, SnowflakeRestClient, SnowflakeRestConfig};
 use serde::de::value::MapDeserializer;
@@ -50,7 +49,7 @@ use jetty_core::{
     jetty::{ConnectorConfig, CredentialsMap},
 };
 
-use anyhow::{anyhow, Context, Result};
+use anyhow::{anyhow, bail, Context, Result};
 use async_trait::async_trait;
 use serde::{Deserialize, Deserializer};
 use serde_json::Value as JsonValue;
@@ -63,6 +62,44 @@ const CONCURRENT_WAREHOUSE_QUERIES: usize = 5;
 pub struct SnowflakeConnector {
     rest_client: SnowflakeRestClient,
     client: connectors::ConnectorClient,
+    config: SnowflakeConnectorConfig,
+}
+
+/// The configuration values from the jetty_config entry for the connector
+#[derive(Debug, Deserialize)]
+#[serde(deny_unknown_fields)]
+pub(crate) struct SnowflakeConnectorConfig {
+    include: Option<HashSet<String>>,
+}
+
+/// Given an ConnectorConfig object, return a SnowflakeConnectorConfig object.
+/// Throws an error on unexpected fields.
+fn parse_connector_config(connector_config: &ConnectorConfig) -> Result<SnowflakeConnectorConfig> {
+    let config = serde_json::to_value(connector_config.config.clone())?;
+    let mut parsed_config = serde_json::from_value(config)
+        .context("Failed to parse Snowflake connector configuration")?;
+
+    if let SnowflakeConnectorConfig {
+        include: Some(include_set),
+    } = parsed_config
+    {
+        parsed_config.include = Some(expand_include_set(include_set));
+    };
+
+    Ok(parsed_config)
+}
+
+fn expand_include_set(include_set: HashSet<String>) -> HashSet<String> {
+    let mut expanded_include = HashSet::new();
+    for include_name in include_set {
+        let name_parts = include_name.split('.').collect::<Vec<_>>();
+        for i in 1..name_parts.len() + 1 {
+            let prefix = name_parts[0..i].join(".").to_string();
+            expanded_include.insert(prefix);
+        }
+    }
+
+    expanded_include
 }
 
 #[derive(Deserialize, Debug)]
@@ -79,7 +116,7 @@ impl NewConnector for SnowflakeConnector {
     /// Snowflake. Stashes the credentials in the struct for use when
     /// connecting.
     async fn new(
-        _config: &ConnectorConfig,
+        config: &ConnectorConfig,
         credentials: &CredentialsMap,
         connector_client: Option<connectors::ConnectorClient>,
         _data_dir: Option<PathBuf>,
@@ -123,6 +160,7 @@ impl NewConnector for SnowflakeConnector {
             Ok(Box::new(SnowflakeConnector {
                 client,
                 rest_client: SnowflakeRestClient::new(conn, SnowflakeRestConfig { retry: true })?,
+                config: parse_connector_config(config)?,
             }))
         }
     }
@@ -296,25 +334,27 @@ impl Connector for SnowflakeConnector {
             }
         }
         Ok(format!(
-            "{} successful queries\n{} failed queries",
-            success_counter, failure_counter
+            "{success_counter} successful queries\n{failure_counter} failed queries"
         ))
     }
 }
 
 impl SnowflakeConnector {
     /// Get all grants to a role – the privileges and "children" roles.
-    pub(crate) async fn get_grants_to_role_future(
+    pub(crate) async fn get_privilege_grants_future(
         &self,
-        role: &Role,
         target: Arc<Mutex<&mut Vec<StandardGrant>>>,
     ) -> Result<()> {
-        let RoleName(role_name) = &role.name;
         let res = self
-            .query_to_obj::<StandardGrant>(&format!("SHOW GRANTS TO ROLE \"{}\"", &role_name))
+            .query_to_obj::<StandardGrant>("select * from snowflake.account_usage.grants_to_roles where deleted_on is null and granted_on in ('TABLE', 'DATABASE', 'SCHEMA', 'VIEW');")
             .await
-            .context(format!("failed to get grants to role {role_name}"))?;
+            .map_err(|e| {
+                error!("failed to get privilege grants -- error: {}", &e);
+                e
+            })
+            .context("failed to get privilege grants")?;
 
+        debug!("fetched {} privilege grants", res.len());
         let mut target = target.lock().unwrap();
         target.extend(res);
         Ok(())
@@ -345,7 +385,7 @@ impl SnowflakeConnector {
     ) -> Result<()> {
         let res = self
             .query_to_obj::<FutureGrant>(&format!(
-                "SHOW FUTURE GRANTS IN SCHEMA {}.{}",
+                r#"SHOW FUTURE GRANTS IN SCHEMA "{}"."{}""#,
                 &schema.database_name, &schema.name
             ))
             .await
@@ -401,10 +441,17 @@ impl SnowflakeConnector {
 
     /// Get all databases.
     pub async fn get_databases_future(&self, target: &mut Vec<Database>) -> Result<()> {
-        *target = self
+        // here is where we check the include list if it's not blank
+        let mut databases = self
             .query_to_obj::<Database>("SHOW DATABASES")
             .await
             .context("failed to get databases")?;
+
+        if self.config.include.is_some() {
+            databases.retain(|db| self.include_asset(&db.name));
+        }
+
+        *target = databases;
         Ok(())
     }
 
@@ -417,10 +464,17 @@ impl SnowflakeConnector {
 
     /// Get all schemas.
     pub async fn get_schemas_future(&self, target: &mut Vec<Schema>) -> Result<()> {
-        *target = self
+        let mut schemas = self
             .query_to_obj::<Schema>("SHOW SCHEMAS IN ACCOUNT")
             .await
             .context("failed to get schemas")?;
+
+        if self.config.include.is_some() {
+            schemas.retain(|schema| self.include_asset(&schema.fqn()));
+        }
+
+        *target = schemas;
+
         Ok(())
     }
 
@@ -434,10 +488,15 @@ impl SnowflakeConnector {
             "SHOW OBJECTS IN SCHEMA \"{}\".\"{}\"",
             &schema.database_name, &schema.name
         );
-        let res = self
+        let mut res = self
             .query_to_obj::<Object>(&query)
             .await
             .context("failed to get tables")?;
+
+        if self.config.include.is_some() {
+            res.retain(|object| self.include_asset(&object.fqn()));
+        }
+
         let mut target = target.lock().unwrap();
         target.extend(res);
         Ok(())
@@ -446,7 +505,7 @@ impl SnowflakeConnector {
     /// Execute the given query and deserialize the result into the given type.
     pub async fn query_to_obj<T>(&self, query: &str) -> Result<Vec<T>>
     where
-        T: for<'de> Deserialize<'de>,
+        T: for<'de> Deserialize<'de> + std::fmt::Debug,
     {
         let result = self
             .rest_client
@@ -454,54 +513,96 @@ impl SnowflakeConnector {
                 sql: query.to_string(),
                 use_jwt: self.client != connectors::ConnectorClient::Test,
             })
-            .await
-            .context("query failed")?;
+            .await;
+
+        let result = match result {
+            Ok(s) => s,
+            Err(e) => {
+                error!("error running `{query}`: {e}");
+                bail!("error running `{query}`: {e}");
+            }
+        };
+
         if result.is_empty() {
             // TODO: Determine whether this is actually okay behavior.
             return Ok(vec![]);
         }
 
-        let rows_value: JsonValue =
-            serde_json::from_str(&result).context("failed to deserialize")?;
-        if let Some(info) = rows_value.get("partitionInfo") {
-            panic!("Unexpected partitioned return value: {info}");
-        }
-        let rows_data = rows_value["data"].clone();
-        let rows = serde_json::from_value::<Vec<Vec<Option<String>>>>(rows_data)
-            .context("failed to deserialize rows")?
-            .into_iter()
-            .map(|v| v.iter().map(|f| f.clone().unwrap_or_default()).collect());
+        let rows_value: JsonValue = serde_json::from_str(&result)
+            .context("failed to deserialize")
+            .map_err(|e| {
+                error!(
+                    "failed to deserialize result for query: {query} -- error: {}",
+                    &e
+                );
+                e
+            })?;
+
+        // Get column information. Only provided by the first partition
         let fields_intermediate: Vec<SnowflakeField> =
             serde_json::from_value(rows_value["resultSetMetaData"]["rowType"].clone())
+                .map_err(|e| {
+                    error!(
+                        "failed to deserialize fields for query: {query} -- error: {}",
+                        &e
+                    );
+                    e
+                })
                 .context("failed to deserialize fields")?;
         let fields: Vec<String> = fields_intermediate.iter().map(|i| i.name.clone()).collect();
-        Ok(rows
-            .map(|i: Vec<_>| {
-                // Zip field - i
-                let vals: HashMap<String, String> = zip(fields.clone(), i).collect();
-                T::deserialize(MapDeserializer::<
-                    std::collections::hash_map::IntoIter<std::string::String, std::string::String>,
-                    serde::de::value::Error,
-                >::new(vals.into_iter()))
-                .context("couldn't deserialize")
-                .unwrap()
-            })
-            .collect())
+
+        if let Some(partition_info) = rows_value
+            .get("resultSetMetaData")
+            .and_then(|v| v.get("partitionInfo"))
+        {
+            let statement_handle = rows_value
+                .get("statementHandle")
+                .and_then(|v| v.as_str())
+                .expect("partitioned results must have a statement handle");
+            let partition_count = partition_info
+                .as_array()
+                .expect("partitionInfo must be an array")
+                .len();
+            let mut results = value_to_vector(&rows_value, query, &fields)?;
+            for current_partition in 1..partition_count {
+                let partition_row_values: JsonValue = self
+                    .rest_client
+                    .get_partition(
+                        &SnowflakeRequestConfig {
+                            sql: Default::default(),
+                            use_jwt: self.client != connectors::ConnectorClient::Test,
+                        },
+                        statement_handle,
+                        current_partition,
+                    )?
+                    .send()
+                    .await?
+                    .json()
+                    .await?;
+                results.extend(value_to_vector(&partition_row_values, query, &fields)?);
+            }
+            Ok(results)
+        } else {
+            value_to_vector(&rows_value, query, &fields)
+        }
     }
 
     fn grants_to_policies(&self, grants: &[GrantType]) -> Vec<nodes::RawPolicy> {
+        let filter_to_include_list = self.config.include.is_some();
         grants
             .iter()
-            .filter(|g| consts::ASSET_TYPES.contains(&g.granted_on()))
+            .filter(|g| {
+                consts::ASSET_TYPES.contains(&g.granted_on())
+                    && (!filter_to_include_list || self.include_asset(&g.granted_on_name()))
+            })
             // Collect roles by asset name so the role:asset ratio is 1:1.
             .fold(
                 HashMap::new(),
                 |mut asset_map: HashMap<String, HashSet<GrantType>>, g| {
-                    if let Some(asset_privileges) = asset_map.get_mut(g.granted_on_name()) {
+                    if let Some(asset_privileges) = asset_map.get_mut(&g.granted_on_name()) {
                         asset_privileges.insert(g.clone());
                     } else {
-                        asset_map
-                            .insert(g.granted_on_name().to_owned(), HashSet::from([g.clone()]));
+                        asset_map.insert(g.granted_on_name(), HashSet::from([g.clone()]));
                     }
                     asset_map
                 },
@@ -533,10 +634,14 @@ impl SnowflakeConnector {
         &self,
         grants: &[FutureGrant],
     ) -> Vec<nodes::RawDefaultPolicy> {
+        let filter_to_include_list = self.config.include.is_some();
         grants
             .iter()
             // filter down to the asset types we support
-            .filter(|g| consts::ASSET_TYPES.contains(&g.grant_on()))
+            .filter(|g| {
+                consts::ASSET_TYPES.contains(&g.grant_on())
+                    && (!filter_to_include_list || self.include_asset(g.root_asset()))
+            })
             // Collect policies by asset name and grant_on (asset type). Asset type and role combined give a path, so this will give us a single policy
             // for each combo of (Asset, Path, Asset Type, and Agent)
             .fold(
@@ -575,11 +680,64 @@ impl SnowflakeConnector {
             })
             .collect::<Vec<_>>()
     }
+
+    fn include_asset(&self, asset_name: &str) -> bool {
+        let include_paths = match self.config.include {
+            Some(ref paths) => paths,
+            // If there are no include paths, we include everything.
+            None => return true,
+        };
+
+        include_paths.iter().any(|include_path| {
+            if include_path.ends_with('*') {
+                asset_name.starts_with(&include_path[..include_path.len() - 1])
+            } else {
+                include_path == asset_name
+            }
+        })
+    }
+}
+
+fn value_to_vector<T>(value: &JsonValue, query: &str, fields: &[String]) -> Result<Vec<T>>
+where
+    T: for<'de> Deserialize<'de> + std::fmt::Debug,
+{
+    let rows_data = value["data"].clone();
+    let rows = serde_json::from_value::<Vec<Vec<Option<String>>>>(rows_data)
+        .map_err(|e| {
+            error!(
+                "failed to deserialize rows for query: {query} -- error: {}",
+                &e
+            );
+            e
+        })
+        .context("failed to deserialize rows")?
+        .into_iter()
+        .map(|v| v.iter().map(|f| f.clone().unwrap_or_default()).collect());
+    Ok(rows
+        .map(|i: Vec<_>| {
+            // Zip field - i
+            let vals: HashMap<String, String> = zip(fields.to_owned(), i).collect();
+            T::deserialize(MapDeserializer::<
+                std::collections::hash_map::IntoIter<std::string::String, std::string::String>,
+                serde::de::value::Error,
+            >::new(vals.into_iter()))
+            .context("couldn't deserialize")
+            .map_err(|e| {
+                error!(
+                    "failed to deserialize final results for query: {query} -- error: {}",
+                    &e
+                );
+                e
+            })
+            .unwrap()
+        })
+        .collect())
 }
 
 pub(crate) fn strip_snowflake_quotes(object: String, capitalize: bool) -> String {
     if object.starts_with("\"\"\"") {
-        object.replace("\"\"\"", "\"")
+        object.replace("\"\"\"", "\"\"")
     } else if object.starts_with('"') {
         // Remove the quotes and return the contained part as-is.
         object.trim_matches('"').to_owned()
@@ -595,6 +753,15 @@ pub(crate) fn strip_snowflake_quotes(object: String, capitalize: bool) -> String
     }
 }
 
+/// Given a snowflake identifier (e.g. a table name, but not a fqn), escape any quotes in it by converting to double quotes.
+pub(crate) fn escape_snowflake_quotes(identifier: &str) -> String {
+    if identifier.contains('"') {
+        identifier.replace('"', "\"\"")
+    } else {
+        identifier.to_owned()
+    }
+}
+
 /// A Snowflake Asset. Inner value is the fully-qualified snowflake name.
 #[derive(PartialEq, Debug)]
 enum SnowflakeAsset {
@@ -633,3 +800,111 @@ where
     let buf = String::deserialize(deserializer)?;
     Ok(strip_snowflake_quotes(buf, false))
 }
+
+#[cfg(test)]
+mod test {
+    use crate::creds::SnowflakeCredentials;
+
+    use super::*;
+
+    async fn new_snowflake_connector() -> SnowflakeConnector {
+        let config = &ConnectorConfig {
+            connector_type: "snowflake".to_owned(),
+            config: [].into(),
+        };
+        let creds = &jetty_core::fetch_credentials(jetty_core::project::connector_cfg_path())
+            .unwrap()["snowflake"];
+        let snow = *SnowflakeConnector::new(config, creds, None, None)
+            .await
+            .unwrap();
+        snow
+    }
+
+    #[test]
+    fn test_include_set_expansion() -> Result<()> {
+        let include_set = ["A.B*", "D", "E*", "H.I", "F.G.J", "X.*"]
+            .iter()
+            .map(|s| s.to_string())
+            .collect();
+        let new_set = expand_include_set(include_set);
+        assert_eq!(
+            new_set,
+            ["A.B*", "A", "D", "E*", "F", "F.G", "F.G.J", "H", "H.I", "X.*", "X"]
+                .iter()
+                .map(|s| s.to_string())
+                .collect::<HashSet<String>>()
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn test_include() -> Result<()> {
+        let include_set = ["A.B*", "D", "E*", "H.I", "F.G.J", "X.*"]
+            .iter()
+            .map(|s| s.to_string())
+            .collect();
+        let new_set = expand_include_set(include_set);
+
+        let creds = SnowflakeCredentials {
+            account: "my_account".to_owned(),
+            role: "role".to_owned(),
+            user: "user".to_owned(),
+            warehouse: "warehouse".to_owned(),
+            private_key: "private_key".to_owned(),
+            public_key_fp: "fp".to_owned(),
+            url: None,
+        };
+
+        let conn = SnowflakeConnector {
+            config: SnowflakeConnectorConfig {
+                include: Some(new_set),
+            },
+            rest_client: SnowflakeRestClient::new(creds, SnowflakeRestConfig::default()).unwrap(),
+            client: connectors::ConnectorClient::Test,
+        };
+
+        assert!(conn.include_asset("A.B.C"));
+        assert!(conn.include_asset("A.BC"));
+        assert!(!conn.include_asset("D.A"));
+        assert!(conn.include_asset("F"));
+        assert!(conn.include_asset("F.G"));
+        assert!(conn.include_asset("F.G.J"));
+        assert!(!conn.include_asset("F.G.A"));
+        assert!(!conn.include_asset("Z.G.A"));
+        assert!(conn.include_asset("X"));
+        assert!(conn.include_asset("X.X"));
+
+        Ok(())
+    }
+
+    impl SnowflakeConnector {
+        async fn test_get_privilege_grants_slow(
+            &self,
+            target: Arc<Mutex<&mut Vec<StandardGrant>>>,
+        ) -> Result<()> {
+            let res = self
+                .query_to_obj::<StandardGrant>("select * from snowflake.account_usage.grants_to_roles left join (SELECT COUNT(seq4()) as bob FROM TABLE(GENERATOR(TIMELIMIT => 60)) v) on TRUE  where deleted_on is null and granted_on in ('TABLE', 'DATABASE', 'SCHEMA', 'VIEW');")
+                .await
+                .map_err(|e| {
+                    error!("failed to get privilege grants -- error: {}", &e);
+                    e
+                })
+                .context("failed to get privilege grants")?;
+
+            let mut target = target.lock().unwrap();
+            target.extend(res);
+            Ok(())
+        }
+    }
+
+    #[ignore = "slow live query test"]
+    #[tokio::test]
+    async fn test_get_privilege_grants() -> Result<()> {
+        let snow = new_snowflake_connector().await;
+        let mut grants = vec![];
+        snow.test_get_privilege_grants_slow(Arc::new(Mutex::new(&mut grants)))
+            .await?;
+        println!("{} grants", grants.len());
+        Ok(())
+    }
+}
