@@ -9,20 +9,18 @@ use async_trait::async_trait;
 use futures::StreamExt;
 use futures::{join, Future};
 use jetty_core::cual::Cual;
-use jetty_core::logging::error;
+use jetty_core::logging::{error, warn};
 use serde::{Deserialize, Serialize};
 use serde_with::serde_as;
 
-use crate::file_parse::origin::SourceOrigin;
 use crate::nodes::{self, Permissionable, ProjectId, TableauCualable};
 
+use crate::origin::SourceOrigin;
 use crate::rest::{self, TableauAssetType};
 use crate::TableauCredentials;
 
-/// Number of assets to download concurrently
-const CONCURRENT_ASSET_DOWNLOADS: usize = 25;
 /// Number of metadata request to run currently (e.g. permissions)
-pub(crate) const CONCURRENT_METADATA_FETCHES: usize = 100;
+pub(crate) const CONCURRENT_METADATA_FETCHES: usize = 50;
 /// Path to serialized version of the Tableau Env
 const SERIALIZED_ENV_FILENAME: &str = "tableau_env.json";
 
@@ -80,40 +78,7 @@ impl Environment {
 /// Makes it simpler to download these sources
 #[async_trait]
 pub(crate) trait HasSources {
-    /// Get id of asset
-    fn id(&self) -> &String;
-    /// Get name of asset
-    fn name(&self) -> &String;
-    /// Get updated_at
-    fn updated_at(&self) -> &String;
-    /// Get sources
-    fn sources(&self) -> (HashSet<SourceOrigin>, HashSet<SourceOrigin>);
-    /// Fetch sources for an asset
-    async fn fetch_sources(
-        &self,
-        coord: &Coordinator,
-    ) -> Result<(HashSet<SourceOrigin>, HashSet<SourceOrigin>)>;
     fn set_sources(&mut self, sources: (HashSet<SourceOrigin>, HashSet<SourceOrigin>));
-
-    /// Update sources for an asset
-    async fn update_sources<T: HasSources + Sync + Send>(
-        &mut self,
-        coord: &Coordinator,
-        env_assets: &HashMap<String, T>,
-    ) -> Result<()> {
-        let id = self.id().to_owned();
-        match env_assets.get(&id) {
-            Some(old_asset) if old_asset.updated_at() == self.updated_at() => {
-                self.set_sources(old_asset.sources());
-                anyhow::Ok(())
-            }
-            _ => {
-                let x = self.fetch_sources(coord);
-                self.set_sources(x.await?);
-                Ok(())
-            }
-        }
-    }
 }
 
 /// Coordinator handles manages and updates the connector's representation
@@ -161,17 +126,6 @@ impl Coordinator {
         })
     }
 
-    /// Create a dummy coordinator without a working rest client. The environment will be read from
-    /// a file, if available, but cannot be updated.
-    #[cfg(test)]
-    pub(crate) fn new_dummy() -> Self {
-        Coordinator {
-            env: Default::default(),
-            rest_client: rest::TableauRestClient::new_dummy(),
-            data_dir: None,
-        }
-    }
-
     /// Get current environment state from Tableau Online
     pub(crate) async fn update_env(&mut self) -> Result<()> {
         // Fetch all the basic resources. Make them into an iterable to make it easier to run concurrently
@@ -228,22 +182,6 @@ impl Coordinator {
             cual_id_map: Default::default(),
         };
 
-        // FUTURE: put this back in. Right now, we don't fetch the sources well, so we'll just skip it. This needs to be
-        // reworked to use the metadata api
-        // // Now, make sure that assets sources are all up to date
-
-        // let source_futures = vec![
-        //     self.get_source_futures_from_map(&mut new_env.flows, &self.env.flows),
-        //     self.get_source_futures_from_map(&mut new_env.datasources, &self.env.datasources),
-        //     self.get_source_futures_from_map(&mut new_env.workbooks, &self.env.workbooks),
-        // ];
-
-        // // Source fetches
-        // futures::stream::iter(source_futures.into_iter().flatten())
-        //     .buffer_unordered(CONCURRENT_ASSET_DOWNLOADS)
-        //     .collect::<Vec<_>>()
-        //     .await;
-
         // Clone the env so we don't try to both immutably and mutably borrow at the same time.
         let new_env_clone = new_env.clone();
         // Now update permissions. NOTE: This must happen AFTER getting groups and users.
@@ -300,6 +238,11 @@ impl Coordinator {
         // update self.env
         self.env = new_env;
 
+        // update the lineage
+        if let Err(e) = self.update_lineage().await {
+            warn!("problem updating Tableau lineage: {e}");
+        };
+
         // serialize as JSON
         if let Some(dir) = &self.data_dir {
             let file_path = dir.join(SERIALIZED_ENV_FILENAME);
@@ -325,29 +268,6 @@ impl Coordinator {
         fetches
     }
 
-    /// Return a Vec of futures (sort of - look at return type) that will fetch futures from
-    /// a map of assets
-    #[allow(clippy::type_complexity)]
-    fn get_source_futures_from_map<'a, T: HasSources + Send + Sync>(
-        &'a self,
-        new_assets: &'a mut HashMap<String, T>,
-        old_assets: &'a HashMap<String, T>,
-    ) -> Vec<
-        Pin<
-            Box<
-                dyn futures::Future<Output = std::result::Result<(), anyhow::Error>>
-                    + std::marker::Send
-                    + '_,
-            >,
-        >,
-    > {
-        let fetches = new_assets
-            .values_mut()
-            .map(|d| d.update_sources(self, old_assets))
-            .collect::<Vec<_>>();
-        fetches
-    }
-
     /// Return a Vec of futures that will request permissions for a collection of assets
     #[allow(clippy::type_complexity)]
     fn get_permission_futures_from_map<'a, T: Permissionable + Send>(
