@@ -96,7 +96,13 @@ pub(crate) trait Permissionable: core::fmt::Debug {
         let permissions_array = rest::get_json_from_path(
             &resp,
             &vec!["permissions".to_owned(), "granteeCapabilities".to_owned()],
-        )?;
+        );
+
+        let permissions_array = match permissions_array {
+            Ok(v) => v,
+            // if there are no permissions, we can just skip this part
+            Err(_) => return Ok(()),
+        };
 
         let final_permissions = if matches!(permissions_array, serde_json::Value::Array(_)) {
             let permissions: Vec<SerializedPermission> = serde_json::from_value(permissions_array)?;
@@ -287,7 +293,7 @@ impl TableauCualable for Lens {
 }
 
 /// Helper struct for deserializing Tableau assets
-#[derive(Deserialize, Debug, Clone)]
+#[derive(Deserialize, Debug, Clone, Default)]
 struct IdField {
     id: String,
 }
@@ -340,21 +346,6 @@ pub(crate) struct Permission {
     pub(crate) capabilities: HashMap<String, String>,
 }
 
-impl Permission {
-    pub(crate) fn has_capability(&self, cap: &str, mode: &str) -> bool {
-        self.capabilities
-            .iter()
-            .any(|(c, m)| c.as_str() == cap && m.as_str() == mode)
-    }
-
-    pub(crate) fn grantee_user_ids(&self) -> Vec<String> {
-        match &self.grantee {
-            Grantee::User(u) => vec![u.id.to_owned()],
-            Grantee::Group(g) => g.includes.clone().iter().map(|u| u.id.to_owned()).collect(),
-        }
-    }
-}
-
 /// Permissions and Jetty policies map 1:1.
 impl From<Permission> for jetty_nodes::RawPolicy {
     /// In order to get a Jetty policy from a permission, we need to grab
@@ -400,16 +391,6 @@ pub(crate) enum Grantee {
     User(tableau_nodes::User),
 }
 
-impl Grantee {
-    /// Get a human-readable name for this grantee.
-    pub(crate) fn get_name(&self) -> &str {
-        match self {
-            Grantee::Group(g) => &g.name,
-            Grantee::User(g) => &g.email,
-        }
-    }
-}
-
 /// Deserialization helper for Tableau permissions
 #[derive(Deserialize, Debug, Clone)]
 struct Capability {
