@@ -3,8 +3,6 @@
 
 mod cual;
 
-use std::io::{Cursor, Read};
-
 use super::*;
 pub(crate) use cual::get_cual_prefix;
 #[cfg(not(test))]
@@ -15,18 +13,11 @@ pub(crate) use cual::{get_tableau_cual, TableauAssetType};
 
 use anyhow::{bail, Context};
 use async_trait::async_trait;
-use bytes::Bytes;
 
+use jetty_core::logging::error;
 use reqwest::Request;
 use serde::Serialize;
 
-pub(crate) trait Downloadable {
-    /// URI path (not including domain, api version, or site id) to download the asset
-    fn get_path(&self) -> String;
-
-    /// a function the unzipper will use to make sure we return the correct file
-    fn match_file(name: &str) -> bool;
-}
 /// Wrapper struct for http functionality
 #[derive(Default)]
 pub struct TableauRestClient {
@@ -61,42 +52,6 @@ impl TableauRestClient {
         self.http_client.execute(request).await
     }
 
-    /// Download a Tableau asset. Most Workbooks and Datasources can have a query parameter to exclude
-    /// extracts. Flows do not have that option, so the bool should be set to false.
-    pub(crate) async fn download<T: Downloadable>(
-        &self,
-        asset: &T,
-        exclude_extracts: bool,
-    ) -> Result<Bytes> {
-        let mut req = self
-            .build_request(asset.get_path(), None, reqwest::Method::GET)?
-            .header("Accept", "application/zip, application/octet-stream");
-
-        if exclude_extracts {
-            req = req.query(&[("includeExtract", "False")]);
-        }
-
-        req.send().await?.bytes().await.context("downloading file")
-    }
-
-    /// Create a dummy client
-    #[cfg(test)]
-    pub(crate) fn new_dummy() -> Self {
-        set_cual_prefix("dummy-server", "dummy-site");
-
-        TableauRestClient {
-            credentials: TableauCredentials {
-                server_name: "dummy-server".to_owned(),
-                site_name: "dummy-site".to_owned(),
-                ..Default::default()
-            },
-            http_client: reqwest::Client::builder().gzip(true).build().unwrap(),
-            token: None,
-            site_id: None,
-            api_version: "3.16".to_owned(),
-        }
-    }
-
     /// Get site_id token from the TableauRestClient.
     pub(crate) fn get_site_id(&self) -> Result<String> {
         Ok(self
@@ -148,6 +103,7 @@ impl TableauRestClient {
             .header("Accept".to_owned(), "application/json".to_owned())
             .send()
             .await?
+            .error_for_status()?
             .json::<serde_json::Value>()
             .await?;
 
@@ -266,29 +222,6 @@ impl TableauRestClient {
     }
 }
 
-/// This function extracts and returns the first file in a zip archive that returns true for name_matcher
-pub(crate) fn unzip_text_file(archive: Bytes, name_matcher: fn(&str) -> bool) -> Result<String> {
-    let archive_cursor = Cursor::new(archive);
-
-    let mut zip_archive = zip::ZipArchive::new(archive_cursor)?;
-
-    let file_names = zip_archive
-        .file_names()
-        .map(|s| s.to_owned())
-        .collect::<Vec<_>>();
-
-    for name in file_names {
-        if name_matcher(&name) {
-            let mut archive_file = zip_archive.by_name(&name)?;
-            let mut data = String::new();
-            archive_file.read_to_string(&mut data)?;
-
-            return Ok(data);
-        }
-    }
-    bail!("unable to find file to parse");
-}
-
 pub(crate) fn get_json_from_path(
     val: &serde_json::Value,
     path: &Vec<String>,
@@ -330,16 +263,56 @@ impl FetchJson for reqwest::RequestBuilder {
         &self,
         path_to_paginated_iterable: Option<Vec<String>>,
     ) -> Result<serde_json::Value> {
-        let resp = self
+        let cloned_req = self
             .try_clone()
-            .ok_or_else(|| anyhow!("unable to clone request"))?
+            .ok_or_else(|| anyhow!("unable to clone request"))?;
+        // Build the request so that we can inspect it in the case of an error
+        let built_req = self.try_clone().map(|r| r.build());
+        let req_method = if let Some(Ok(built_req)) = &built_req {
+            Some(built_req.method().to_owned())
+        } else {
+            None
+        };
+        let req_url = if let Some(Ok(built_req)) = built_req {
+            Some(built_req.url().to_owned())
+        } else {
+            None
+        };
+
+        let resp = cloned_req
             .send()
             .await
-            .context("making request")?;
+            .map_err(|e| {
+                if let (Some(method), Some(url)) = (&req_method, &req_url) {
+                    error!("error with request ({method:?} {url:?}): {e}")
+                } else {
+                    error!("error with request: {e}")
+                };
+                e
+            })
+            .context("making request")?
+            .error_for_status()
+            .map_err(|e| {
+                if let (Some(method), Some(url)) = (&req_method, &req_url) {
+                    error!("error with request - bad response ({method:?} {url:?}): {e}")
+                } else {
+                    error!("error with request - bad response: {e}")
+                };
+                e
+            })?;
 
         let parsed_response = resp
             .json::<serde_json::Value>()
             .await
+            .map_err(|e| {
+                let request = if let (Some(method), Some(url)) = (&req_method, &req_url) {
+                    format!("({method:?} {url:?})")
+                } else {
+                    "".to_owned()
+                };
+                error!("error parsing json response {request}: {e}");
+                e
+            })
             .context("parsing json response")?;
 
         // Check for pagination
@@ -393,6 +366,15 @@ impl FetchJson for reqwest::RequestBuilder {
                     .send()
                     .await
                     .context("making request")?
+                    .error_for_status()
+                    .map_err(|e| {
+                        if let (Some(method), Some(url)) = (&req_method, &req_url) {
+                            error!("error with pagination request - bad response ({method:?} {url:?}): {e}")
+                        } else {
+                            error!("error with pagination request - bad response: {e}")
+                        };
+                        e
+                    })?
                     .json::<serde_json::Value>()
                     .await
                     .context("parsing json response")?;
